==> imcode-ldap-1.0-20170525.093657-1-sources.jar: com/imcode/net/ldap/LdapConnectionImpl.java <==
package com.imcode.net.ldap;

import org.apache.commons.collections.map.CaseInsensitiveMap;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.UnhandledException;
import org.apache.log4j.Logger;

import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.LdapName;
import java.util.*;

public abstract class LdapConnectionImpl implements LdapConnection {

    private static final Logger LOG = Logger.getLogger(LdapConnectionImpl.class);

    static final String AUTHENTICATION_TYPE_SIMPLE = "simple";
    private static final String EMPTY_NAME = "";

    private final LdapContext ldapContext;
    private final Date expiryDate;

    public LdapConnectionImpl(LdapContext ldapContext, Date expiryDate) {
        this.ldapContext = ldapContext;
        this.expiryDate = expiryDate;
    }

    public Iterator<Map<String, String>> search(String searchFilterExpr, Object[] parameters,
                                                SearchControls searchControls) throws LdapClientException {
        try {
            try {
                return trySearch(searchFilterExpr, parameters, searchControls);
            } catch (CommunicationException ce) {
                LOG.warn("Problem communicating with LDAP server, retrying.", ce);
                return trySearch(searchFilterExpr, parameters, searchControls);
            }
        } catch (NamingException ne) {
            throw new LdapClientException("LDAP search failed.", ne);
        }
    }

    public Iterator<Map<String, Set<String>>> searchMultivalues(String searchFilterExpr, Object[] parameters,
                                                                SearchControls searchControls) throws LdapClientException {
        try {
            try {
                return trySearchMultivalues(searchFilterExpr, parameters, searchControls);
            } catch (CommunicationException ce) {
                LOG.warn("Problem communicating with LDAP server, retrying.", ce);
                return trySearchMultivalues(searchFilterExpr, parameters, searchControls);
            }
        } catch (NamingException ne) {
            throw new LdapClientException("LDAP search failed.", ne);
        }
    }

    public Set<String> getADUserGroups(final String sAMAccountName) throws LdapClientException {
        String userDn = getADUserDn(sAMAccountName);

        if (userDn == null) return Collections.emptySet();

        Set<String> groups = new HashSet<String>();
        SearchControls userSearchCtls = new SearchControls();

        userSearchCtls.setSearchScope(SearchControls.OBJECT_SCOPE);
        userSearchCtls.setReturningAttributes(new String[]{"tokenGroups"});

        try {
            NamingEnumeration userAnswer = null;
            NamingEnumeration groupsAnswer = null;
            try {
                Name userDnName = new LdapName(userDn);
                Name ctxName = new LdapName(ldapContext.getNameInNamespace());
                Name name = userDnName.getSuffix(ctxName.size());
                userAnswer = ldapContext.search(name, "(objectClass=user)", userSearchCtls);

                StringBuilder groupsSearchFilter = new StringBuilder();

                groupsSearchFilter.append("(|");

                while (userAnswer.hasMoreElements()) {
                    SearchResult sr = (SearchResult) userAnswer.nextElement();
                    Attributes attrs = sr.getAttributes();

                    if (attrs != null) {
                        for (NamingEnumeration ae = attrs.getAll(); ae.hasMoreElements(); ) {
                            Attribute attr = (Attribute) ae.nextElement();
                            for (NamingEnumeration e = attr.getAll(); e.hasMoreElements(); ) {
                                byte[] sid = (byte[]) e.nextElement();
                                groupsSearchFilter.append("(objectSid=" + binarySidToStringSid(sid) + ")");
                            }
                        }
                    }
                }

                groupsSearchFilter.append(")");

                SearchControls groupsSearchCtls = new SearchControls();
                groupsSearchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
                groupsSearchCtls.setReturningAttributes(new String[]{"sAMAccountName"});

                groupsAnswer = ldapContext.search(EMPTY_NAME, groupsSearchFilter.toString(), groupsSearchCtls);

                while (groupsAnswer.hasMoreElements()) {
                    SearchResult sr = (SearchResult) groupsAnswer.nextElement();
                    Attributes attrs = sr.getAttributes();

                    if (attrs != null) {
                        groups.add(attrs.get("sAMAccountName").get().toString());
                    }
                }
            } finally {
                if (userAnswer != null) {
                    userAnswer.close();
                }
                if (groupsAnswer != null) {
                    groupsAnswer.close();
                }
            }

        } catch (NamingException e) {
            throw new LdapClientException("", e);
        }

        return groups;
    }

    /**
     * @param sAMAccountName
     * @return Active directory user's DN or null if there is no user with provided sAMAAccountName.
     * @throws LdapClientException
     */
    private String getADUserDn(final String sAMAccountName) throws LdapClientException {
        return executeWitReconnect(new LdapCommand<String>() {
            @Override
            public String execute() throws NamingException, LdapClientException {
                String searchFilter = "(&(objectClass=user)(sAMAccountName={0}))";
                SearchControls searchCtls = new SearchControls();
                searchCtls.setReturningAttributes(new String[]{});
                searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

                NamingEnumeration<SearchResult> result = null;
                try {
                    result = ldapContext.search(EMPTY_NAME, searchFilter, new Object[]{sAMAccountName}, searchCtls);

                    String dn = null;

                    if (result.hasMoreElements()) {
                        dn = result.nextElement().getNameInNamespace();
                    }

                    return dn;
                } finally {
                    if (result != null) {
                        result.close();
                    }
                }

            }
        });
    }

    public boolean isExpired() {
        return new Date().after(expiryDate);
    }

    private static class SearchResultIterator implements Iterator<Map<String, String>> {

        private final NamingEnumeration<SearchResult> enumeration;
        private final SearchControls searchControls;

        SearchResultIterator(NamingEnumeration<SearchResult> enumeration, SearchControls searchControls) {
            this.enumeration = enumeration;
            this.searchControls = searchControls;
        }

        public boolean hasNext() {
            final boolean hasMoreElements = enumeration.hasMoreElements();
            if (!hasMoreElements) {
                try {
                    enumeration.close();
                } catch (NamingException e) {
                    throw new UnhandledException(e);
                }
            }
            return hasMoreElements;
        }

        public Map<String, String> next() {
            try {
                return createMapFromSearchResult(enumeration.nextElement());
            } catch (NamingException e) {
                throw new UnhandledException(e);
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        private Map<String, String> createMapFromSearchResult(SearchResult searchResult) throws NamingException {

            NamingEnumeration attribEnum = null;
            try {
                attribEnum = searchResult.getAttributes().getAll();
                Map<String, String> attributes = new CaseInsensitiveMap();
                while (attribEnum.hasMoreElements()) {
                    Attribute attribute = (Attribute) attribEnum.nextElement();
                    String attributeName = attribute.getID();
                    String attributeValue = attribute.get().toString();
                    attributes.put(attributeName, attributeValue);
                }
                if (!attributes.containsKey(DISTINGUISHED_NAME)) {
                    boolean includeDistinguishedName = null != searchControls && searchControls.getReturningObjFlag() && (null == searchControls.getReturningAttributes()
                            || ArrayUtils.contains(searchControls.getReturningAttributes(), DISTINGUISHED_NAME));
                    if (includeDistinguishedName) {
                        DirContext dirContext = null;
                        try {
                            dirContext = (DirContext) searchResult.getObject();
                            String distinguishedName = dirContext.getNameInNamespace();
                            attributes.put(DISTINGUISHED_NAME, distinguishedName);
                        } finally {
                            if (dirContext != null) {
                                dirContext.close();
                            }
                        }
                    }
                }
                return attributes;
            } finally {
                if (attribEnum != null) {
                    attribEnum.close();
                }
            }
        }

    }


    private static class SearchMultivalueResultIterator implements Iterator<Map<String, Set<String>>> {

        private final NamingEnumeration<SearchResult> enumeration;
        private final SearchControls searchControls;

        SearchMultivalueResultIterator(NamingEnumeration<SearchResult> enumeration, SearchControls searchControls) {
            this.enumeration = enumeration;
            this.searchControls = searchControls;
        }

        public boolean hasNext() {
            final boolean hasMoreElements = enumeration.hasMoreElements();
            if (!hasMoreElements) {
                try {
                    enumeration.close();
                } catch (NamingException e) {
                    throw new UnhandledException(e);
                }
            }
            return hasMoreElements;
        }

        public Map<String, Set<String>> next() {
            try {
                return createMapFromSearchResult(enumeration.nextElement());
            } catch (NamingException e) {
                throw new UnhandledException(e);
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        private Map<String, Set<String>> createMapFromSearchResult(SearchResult searchResult) throws NamingException {

            NamingEnumeration attribEnum = null;
            try {
                attribEnum = searchResult.getAttributes().getAll();
                Map<String, Set<String>> attributes = new CaseInsensitiveMap();
                while (attribEnum.hasMoreElements()) {
                    Attribute attribute = (Attribute) attribEnum.nextElement();
                    String attributeName = attribute.getID();
                    Enumeration<String> attributeValuesEnum = (Enumeration<String>) attribute.getAll();
                    Set<String> attributeValues = new HashSet<String>();

                    while (attributeValuesEnum.hasMoreElements()) {
                        attributeValues.add(attributeValuesEnum.nextElement());
                    }

                    attributes.put(attributeName, Collections.unmodifiableSet(attributeValues));
                }

                return attributes;
            } finally {
                if (attribEnum != null) {
                    attribEnum.close();
                }
            }

        }

    }



    private static String binarySidToStringSid(byte[] SID) {
        String strSID = "";

        long version;
        long authority;
        long count;
        long rid;

        strSID = "S";
        version = SID[0];
        strSID = strSID + "-" + Long.toString(version);
        authority = SID[4];

        for (int i = 0; i < 4; i++) {
            authority <<= 8;
            authority += SID[4 + i] & 0xFF;
        }

        strSID = strSID + "-" + Long.toString(authority);
        count = SID[2];
        count <<= 8;
        count += SID[1] & 0xFF;

        for (int j = 0; j < count; j++) {
            rid = SID[11 + (j * 4)] & 0xFF;

            for (int k = 1; k < 4; k++) {
                rid <<= 8;
                rid += SID[11 - k + (j * 4)] & 0xFF;
            }

            strSID = strSID + "-" + Long.toString(rid);
        }

        return strSID;
    }

    private interface LdapCommand<T> {
        T execute() throws NamingException, LdapClientException;
    }


    private <T> T executeWitReconnect(LdapCommand<T> ldapCommand) throws LdapClientException {
        return executeWitReconnect(ldapCommand, 2);
    }

    private <T> T executeWitReconnect(LdapCommand<T> ldapCommand, int retryCount) throws LdapClientException {
        T result;

        try {
            result = ldapCommand.execute();
        } catch (CommunicationException e) {
            if (retryCount > 1) {
                result = executeWitReconnect(ldapCommand, retryCount - 1);
            } else {
                throw new LdapClientException("", e);
            }
        } catch (NamingException e) {
            throw new LdapClientException("", e);
        }

        return result;
    }

    private Iterator<Map<String, String>> trySearch(final String searchFilterExpr, final Object[] parameters, SearchControls searchControls) throws NamingException, LdapClientException {
        if (null == searchControls) {
            searchControls = new SearchControls();
        }
        searchControls.setReturningObjFlag(true);

        final NamingEnumeration<SearchResult> enumeration = ldapContext.search(EMPTY_NAME, searchFilterExpr, parameters, searchControls);

        return new SearchResultIterator(enumeration, searchControls);
    }

    private Iterator<Map<String, Set<String>>> trySearchMultivalues(String searchFilterExpr, Object[] parameters, SearchControls searchControls) throws NamingException, LdapClientException {
        if (null == searchControls) {
            searchControls = new SearchControls();
        }
        searchControls.setReturningObjFlag(true);

        final NamingEnumeration<SearchResult> enumeration = ldapContext.search(EMPTY_NAME, searchFilterExpr, parameters, searchControls);

        return new SearchMultivalueResultIterator(enumeration, searchControls);
    }
}

==> imcode-ldap-1.0-20170529.112822-2-sources.jar: com/imcode/net/ldap/LdapConnectionImpl.java <==
package com.imcode.net.ldap;

import org.apache.commons.collections.map.CaseInsensitiveMap;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.UnhandledException;
import org.apache.log4j.Logger;

import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.LdapName;
import java.util.*;

public abstract class LdapConnectionImpl implements LdapConnection {

    private static final Logger LOG = Logger.getLogger(LdapConnectionImpl.class);

    static final String AUTHENTICATION_TYPE_SIMPLE = "simple";
    private static final String EMPTY_NAME = "";

    private final LdapContext ldapContext;
    private final Date expiryDate;

    public LdapConnectionImpl(LdapContext ldapContext, Date expiryDate) {
        this.ldapContext = ldapContext;
        this.expiryDate = expiryDate;
    }

    public Iterator<Map<String, String>> search(String searchFilterExpr, Object[] parameters,
                                                SearchControls searchControls) throws LdapClientException {
        try {
            try {
                return trySearch(searchFilterExpr, parameters, searchControls);
            } catch (CommunicationException ce) {
                LOG.warn("Problem communicating with LDAP server, retrying.", ce);
                return trySearch(searchFilterExpr, parameters, searchControls);
            }
        } catch (NamingException ne) {
            throw new LdapClientException("LDAP search failed.", ne);
        }
    }

    public Iterator<Map<String, Set<String>>> searchMultivalues(String searchFilterExpr, Object[] parameters,
                                                                SearchControls searchControls) throws LdapClientException {
        try {
            try {
                return trySearchMultivalues(searchFilterExpr, parameters, searchControls);
            } catch (CommunicationException ce) {
                LOG.warn("Problem communicating with LDAP server, retrying.", ce);
                return trySearchMultivalues(searchFilterExpr, parameters, searchControls);
            }
        } catch (NamingException ne) {
            throw new LdapClientException("LDAP search failed.", ne);
        }
    }

    public Set<String> getADUserGroups(final String sAMAccountName) throws LdapClientException {
        String userDn = getADUserDn(sAMAccountName);

        if (userDn == null) return Collections.emptySet();

        Set<String> groups = new HashSet<String>();
        SearchControls userSearchCtls = new SearchControls();

        userSearchCtls.setSearchScope(SearchControls.OBJECT_SCOPE);
        userSearchCtls.setReturningAttributes(new String[]{"tokenGroups"});

        try {
            NamingEnumeration userAnswer = null;
            NamingEnumeration groupsAnswer = null;
            try {
                Name userDnName = new LdapName(userDn);
                Name ctxName = new LdapName(ldapContext.getNameInNamespace());
                Name name = userDnName.getSuffix(ctxName.size());
                userAnswer = ldapContext.search(name, "(objectClass=user)", userSearchCtls);

                StringBuilder groupsSearchFilter = new StringBuilder();

                groupsSearchFilter.append("(|");

                while (userAnswer.hasMoreElements()) {
                    SearchResult sr = (SearchResult) userAnswer.nextElement();
                    Attributes attrs = sr.getAttributes();

                    if (attrs != null) {
                        for (NamingEnumeration ae = attrs.getAll(); ae.hasMoreElements(); ) {
                            Attribute attr = (Attribute) ae.nextElement();
                            for (NamingEnumeration e = attr.getAll(); e.hasMoreElements(); ) {
                                byte[] sid = (byte[]) e.nextElement();
                                groupsSearchFilter.append("(objectSid=" + binarySidToStringSid(sid) + ")");
                            }
                        }
                    }
                }

                groupsSearchFilter.append(")");

                SearchControls groupsSearchCtls = new SearchControls();
                groupsSearchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
                groupsSearchCtls.setReturningAttributes(new String[]{"sAMAccountName"});

                groupsAnswer = ldapContext.search(EMPTY_NAME, groupsSearchFilter.toString(), groupsSearchCtls);

                while (groupsAnswer.hasMoreElements()) {
                    SearchResult sr = (SearchResult) groupsAnswer.nextElement();
                    Attributes attrs = sr.getAttributes();

                    if (attrs != null) {
                        groups.add(attrs.get("sAMAccountName").get().toString());
                    }
                }
            } finally {
                if (userAnswer != null) {
                    userAnswer.close();
                }
                if (groupsAnswer != null) {
                    groupsAnswer.close();
                }
            }

        } catch (NamingException e) {
            throw new LdapClientException("", e);
        }

        return groups;
    }

    /**
     * @param sAMAccountName
     * @return Active directory user's DN or null if there is no user with provided sAMAAccountName.
     * @throws LdapClientException
     */
    private String getADUserDn(final String sAMAccountName) throws LdapClientException {
        return executeWitReconnect(new LdapCommand<String>() {
            @Override
            public String execute() throws NamingException, LdapClientException {
                String searchFilter = "(&(objectClass=user)(sAMAccountName={0}))";
                SearchControls searchCtls = new SearchControls();
                searchCtls.setReturningAttributes(new String[]{});
                searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

                NamingEnumeration<SearchResult> result = null;
                try {
                    result = ldapContext.search(EMPTY_NAME, searchFilter, new Object[]{sAMAccountName}, searchCtls);

                    String dn = null;

                    if (result.hasMoreElements()) {
                        dn = result.nextElement().getNameInNamespace();
                    }

                    return dn;
                } finally {
                    if (result != null) {
                        result.close();
                    }
                }

            }
        });
    }

    boolean isExpired() {
        return new Date().after(expiryDate);
    }

    void closeLdapContext() throws LdapClientException {
        try {
            ldapContext.close();
        } catch (NamingException e) {
            throw new LdapClientException("", e);
        }
    }

    private static class SearchResultIterator implements Iterator<Map<String, String>> {

        private final NamingEnumeration<SearchResult> enumeration;
        private final SearchControls searchControls;

        SearchResultIterator(NamingEnumeration<SearchResult> enumeration, SearchControls searchControls) {
            this.enumeration = enumeration;
            this.searchControls = searchControls;
        }

        public boolean hasNext() {
            final boolean hasMoreElements = enumeration.hasMoreElements();
            if (!hasMoreElements) {
                try {
                    enumeration.close();
                } catch (NamingException e) {
                    throw new UnhandledException(e);
                }
            }
            return hasMoreElements;
        }

        public Map<String, String> next() {
            try {
                return createMapFromSearchResult(enumeration.nextElement());
            } catch (NamingException e) {
                throw new UnhandledException(e);
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        private Map<String, String> createMapFromSearchResult(SearchResult searchResult) throws NamingException {

            NamingEnumeration attribEnum = null;
            try {
                attribEnum = searchResult.getAttributes().getAll();
                Map<String, String> attributes = new CaseInsensitiveMap();
                while (attribEnum.hasMoreElements()) {
                    Attribute attribute = (Attribute) attribEnum.nextElement();
                    String attributeName = attribute.getID();
                    String attributeValue = attribute.get().toString();
                    attributes.put(attributeName, attributeValue);
                }
                if (!attributes.containsKey(DISTINGUISHED_NAME)) {
                    boolean includeDistinguishedName = null != searchControls && searchControls.getReturningObjFlag() && (null == searchControls.getReturningAttributes()
                            || ArrayUtils.contains(searchControls.getReturningAttributes(), DISTINGUISHED_NAME));
                    if (includeDistinguishedName) {
                        DirContext dirContext = null;
                        try {
                            dirContext = (DirContext) searchResult.getObject();
                            String distinguishedName = dirContext.getNameInNamespace();
                            attributes.put(DISTINGUISHED_NAME, distinguishedName);
                        } finally {
                            if (dirContext != null) {
                                dirContext.close();
                            }
                        }
                    }
                }
                return attributes;
            } finally {
                if (attribEnum != null) {
                    attribEnum.close();
                }
            }
        }

    }


    private static class SearchMultivalueResultIterator implements Iterator<Map<String, Set<String>>> {

        private final NamingEnumeration<SearchResult> enumeration;
        private final SearchControls searchControls;

        SearchMultivalueResultIterator(NamingEnumeration<SearchResult> enumeration, SearchControls searchControls) {
            this.enumeration = enumeration;
            this.searchControls = searchControls;
        }

        public boolean hasNext() {
            final boolean hasMoreElements = enumeration.hasMoreElements();
            if (!hasMoreElements) {
                try {
                    enumeration.close();
                } catch (NamingException e) {
                    throw new UnhandledException(e);
                }
            }
            return hasMoreElements;
        }

        public Map<String, Set<String>> next() {
            try {
                return createMapFromSearchResult(enumeration.nextElement());
            } catch (NamingException e) {
                throw new UnhandledException(e);
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

        private Map<String, Set<String>> createMapFromSearchResult(SearchResult searchResult) throws NamingException {

            NamingEnumeration attribEnum = null;
            try {
                attribEnum = searchResult.getAttributes().getAll();
                Map<String, Set<String>> attributes = new CaseInsensitiveMap();
                while (attribEnum.hasMoreElements()) {
                    Attribute attribute = (Attribute) attribEnum.nextElement();
                    String attributeName = attribute.getID();
                    Enumeration<String> attributeValuesEnum = (Enumeration<String>) attribute.getAll();
                    Set<String> attributeValues = new HashSet<String>();

                    while (attributeValuesEnum.hasMoreElements()) {
                        attributeValues.add(attributeValuesEnum.nextElement());
                    }

                    attributes.put(attributeName, Collections.unmodifiableSet(attributeValues));
                }

                return attributes;
            } finally {
                if (attribEnum != null) {
                    attribEnum.close();
                }
            }

        }

    }



    private static String binarySidToStringSid(byte[] SID) {
        String strSID = "";

        long version;
        long authority;
        long count;
        long rid;

        strSID = "S";
        version = SID[0];
        strSID = strSID + "-" + Long.toString(version);
        authority = SID[4];

        for (int i = 0; i < 4; i++) {
            authority <<= 8;
            authority += SID[4 + i] & 0xFF;
        }

        strSID = strSID + "-" + Long.toString(authority);
        count = SID[2];
        count <<= 8;
        count += SID[1] & 0xFF;

        for (int j = 0; j < count; j++) {
            rid = SID[11 + (j * 4)] & 0xFF;

            for (int k = 1; k < 4; k++) {
                rid <<= 8;
                rid += SID[11 - k + (j * 4)] & 0xFF;
            }

            strSID = strSID + "-" + Long.toString(rid);
        }

        return strSID;
    }

    private interface LdapCommand<T> {
        T execute() throws NamingException, LdapClientException;
    }


    private <T> T executeWitReconnect(LdapCommand<T> ldapCommand) throws LdapClientException {
        return executeWitReconnect(ldapCommand, 2);
    }

    private <T> T executeWitReconnect(LdapCommand<T> ldapCommand, int retryCount) throws LdapClientException {
        T result;

        try {
            result = ldapCommand.execute();
        } catch (CommunicationException e) {
            if (retryCount > 1) {
                result = executeWitReconnect(ldapCommand, retryCount - 1);
            } else {
                throw new LdapClientException("", e);
            }
        } catch (NamingException e) {
            throw new LdapClientException("", e);
        }

        return result;
    }

    private Iterator<Map<String, String>> trySearch(final String searchFilterExpr, final Object[] parameters, SearchControls searchControls) throws NamingException, LdapClientException {
        if (null == searchControls) {
            searchControls = new SearchControls();
        }
        searchControls.setReturningObjFlag(true);

        final NamingEnumeration<SearchResult> enumeration = ldapContext.search(EMPTY_NAME, searchFilterExpr, parameters, searchControls);

        return new SearchResultIterator(enumeration, searchControls);
    }

    private Iterator<Map<String, Set<String>>> trySearchMultivalues(String searchFilterExpr, Object[] parameters, SearchControls searchControls) throws NamingException, LdapClientException {
        if (null == searchControls) {
            searchControls = new SearchControls();
        }
        searchControls.setReturningObjFlag(true);

        final NamingEnumeration<SearchResult> enumeration = ldapContext.search(EMPTY_NAME, searchFilterExpr, parameters, searchControls);

        return new SearchMultivalueResultIterator(enumeration, searchControls);
    }
}
